@@ -1,6 +1,5 @@
 import sys
 import argparse
-import logging
 import os
 import resource
 
@@ -54,7 +53,6 @@ def daemonize():
         os._exit(0)
 
     # subchild
-    os.umask(0)
     maxfd = get_maxfd()
     closerange(0, maxfd)
 
@@ -64,6 +62,19 @@ def daemonize():
 
 
 def main():
+    import zmq
+    try:
+        zmq_version = [int(part) for part in zmq.__version__.split('.')]
+        if len(zmq_version) < 2:
+            raise ValueError()
+    except (AttributeError, ValueError):
+        print('Unknown PyZQM version - aborting...')
+        sys.exit(0)
+
+    if zmq_version[0] < 13 or (zmq_version[0] == 13 and zmq_version[1] < 1):
+        print('circusd needs PyZMQ >= 13.1.0 to run - aborting...')
+        sys.exit(0)
+
     parser = argparse.ArgumentParser(description='Run some watchers.')
     parser.add_argument('config', help='configuration file', nargs='?')
 
@@ -76,6 +87,10 @@ def main():
         "The location where the logs will be written. The default behavior "
         "is to write to stdout (you can force it by passing '-' to "
         "this option). Takes a filename otherwise."))
+    parser.add_argument("--logger-config", dest="loggerconfig", help=(
+        "The location where a standard Python logger configuration INI, "
+        "JSON or YAML file can be found.  This can be used to override "
+        "the default logging configuration for the arbiter."))
 
     parser.add_argument('--daemon', dest='daemonize', action='store_true',
                         help="Start circusd in the background")
@@ -96,18 +111,14 @@ def main():
     if args.daemonize:
         daemonize()
 
-    # This config call is done to avoid any
-    #     "no handlers could be found for logger"
-    #
-    # error while loding the configuration and setting up the arbiter.
-    # The real logging configuration is done right after via
-    # a configure_logger() call
-    logging.basicConfig()
-
     # From here it can also come from the arbiter configuration
     # load the arbiter from config
     arbiter = Arbiter.load_from_config(args.config)
 
+    # go ahead and set umask early if it is in the config
+    if arbiter.umask is not None:
+        os.umask(arbiter.umask)
+
     pidfile = args.pidfile or arbiter.pidfile or None
     if pidfile:
         pidfile = Pidfile(pidfile)
@@ -121,13 +132,14 @@ def main():
     # configure the logger
     loglevel = args.loglevel or arbiter.loglevel or 'info'
     logoutput = args.logoutput or arbiter.logoutput or '-'
-    configure_logger(logger, loglevel, logoutput)
+    loggerconfig = args.loggerconfig or arbiter.loggerconfig or None
+    configure_logger(logger, loglevel, logoutput, loggerconfig)
 
     # Main loop
     restart = True
     while restart:
         try:
-            arbiter = Arbiter.load_from_config(args.config)
+            arbiter = arbiter or Arbiter.load_from_config(args.config)
             future = arbiter.start()
             restart = False
             if check_future_exception_and_log(future) is None:
@@ -139,6 +151,7 @@ def main():
         except KeyboardInterrupt:
             pass
         finally:
+            arbiter = None
             if pidfile is not None:
                 pidfile.unlink()
     sys.exit(0)
