@@ -5,12 +5,11 @@ import uuid
 
 import zmq
 import zmq.utils.jsonapi as json
-from zmq.utils.strtypes import b
 from zmq.eventloop.zmqstream import ZMQStream
 import tornado
 
 from circus.exc import CallError
-from circus.py3compat import string_types
+from circus.py3compat import string_types, b
 from circus.util import DEFAULT_ENDPOINT_DEALER, get_connection
 
 
@@ -46,6 +45,9 @@ class AsyncCircusClient(object):
 
     def stop(self):
         self.stream.stop_on_recv()
+        # only supported by libzmq >= 3
+        if hasattr(self.socket, 'disconnect'):
+            self.socket.disconnect(self.endpoint)
         self.socket.close()
 
     def send_message(self, command, **props):
@@ -103,6 +105,9 @@ class CircusClient(object):
         self.poller.register(self.socket, zmq.POLLIN)
 
     def stop(self):
+        # only supported by libzmq >= 3
+        if hasattr(self.socket, 'disconnect'):
+            self.socket.disconnect(self.endpoint)
         self.socket.close()
 
     def send_message(self, command, **props):
