@@ -5,14 +5,18 @@ import time
 import warnings
 try:
     import queue as Queue
-    from test.support import captured_output
 except ImportError:
     import Queue  # NOQA
-    from test.test_support import captured_output  # NOQA
+try:
+    from test.support import captured_output
+except ImportError:
+    try:
+        from test.test_support import captured_output  # NOQA
+    except ImportError:
+        captured_output = None  # NOQA
 
 import tornado
 import mock
-from zmq.utils.strtypes import u
 
 from circus import logger
 from circus.process import RUNNING, UNEXISTING
@@ -23,6 +27,7 @@ from circus.tests.support import async_poll_for, EasyTestSuite
 from circus.tests.support import MagicMockFuture
 from circus.util import get_python_version, tornado_sleep
 from circus.watcher import Watcher
+from circus.py3compat import s
 
 warnings.filterwarnings('ignore',
                         module='threading', message='sys.exc_clear')
@@ -37,6 +42,9 @@ class FakeProcess(object):
         self.age = age
         self.stopping = False
 
+    def returncode(self):
+        return 0
+
     def children(self):
         return []
 
@@ -236,7 +244,7 @@ class TestWatcherInitialization(TestCircus):
                 messages.append(m)
             except Queue.Empty:
                 pass
-            data = ''.join(u(m['data']) for m in messages)
+            data = ''.join(s(m['data']) for m in messages)
             if 'XYZ' in data:
                 resp = True
                 break
@@ -336,6 +344,14 @@ class TestWatcherHooks(TestCircus):
     def _stop(self):
         yield self.call("stop", name="test", waiting=True)
 
+    @tornado.gen.coroutine
+    def _stats(self):
+        yield self.call("stats", name="test")
+
+    @tornado.gen.coroutine
+    def _extended_stats(self):
+        yield self.call("stats", name="test", extended=True)
+
     @tornado.gen.coroutine
     def get_status(self):
         resp = yield self.call("status", name="test")
@@ -352,12 +368,15 @@ class TestWatcherHooks(TestCircus):
         events = {'before_start_called': False}
 
         def hook(watcher, arbiter, hook_name, **kwargs):
-            events['before_start_called'] = True
+            events['%s_called' % hook_name] = True
             events['arbiter_in_hook'] = arbiter
 
             if hook_kwargs_test_function is not None:
                 hook_kwargs_test_function(kwargs)
 
+            if hook_name == 'extended_stats':
+                kwargs['stats']['tx'] = 1000
+                return
             if behavior == SUCCESS:
                 return True
             elif behavior == FAILURE:
@@ -380,9 +399,35 @@ class TestWatcherHooks(TestCircus):
             yield arbiter.stop()
             logger.exception = old
 
-        self.assertTrue(events['before_start_called'])
+        self.assertTrue(events['%s_called' % hook_name])
         self.assertEqual(events['arbiter_in_hook'], arbiter)
 
+    @tornado.gen.coroutine
+    def _test_extended_stats(self, extended=False):
+        events = {'extended_stats_called': False}
+
+        def hook(watcher, arbiter, hook_name, **kwargs):
+            events['extended_stats_called'] = True
+
+        old = logger.exception
+        logger.exception = lambda x: x
+
+        hooks = {'extended_stats': (hook, False)}
+        testfile, arbiter = self.run_with_hooks(hooks)
+        yield arbiter.start()
+        try:
+            if extended:
+                yield self._extended_stats()
+            else:
+                yield self._stats()
+            resp_status = yield self.get_status()
+            self.assertEqual(resp_status, 'active')
+        finally:
+            yield arbiter.stop()
+            logger.exception = old
+
+        self.assertEqual(events['extended_stats_called'], extended)
+
     @tornado.testing.gen_test
     def test_before_start(self):
         yield self._test_hooks()
@@ -402,9 +447,10 @@ class TestWatcherHooks(TestCircus):
 
     @tornado.testing.gen_test
     def test_after_start_fails(self):
-        with captured_output('stderr'):
-            yield self._test_hooks(behavior=ERROR, status='stopped',
-                                   hook_name='after_start')
+        if captured_output:
+            with captured_output('stderr'):
+                yield self._test_hooks(behavior=ERROR, status='stopped',
+                                       hook_name='after_start')
 
     @tornado.testing.gen_test
     def test_after_start_false(self):
@@ -439,10 +485,11 @@ class TestWatcherHooks(TestCircus):
 
     @tornado.testing.gen_test
     def test_before_stop_fails(self):
-        with captured_output('stdout'):
-            yield self._test_hooks(behavior=ERROR, status='stopped',
-                                   hook_name='before_stop',
-                                   call=self._stop)
+        if captured_output:
+            with captured_output('stdout'):
+                yield self._test_hooks(behavior=ERROR, status='stopped',
+                                       hook_name='before_stop',
+                                       call=self._stop)
 
     @tornado.testing.gen_test
     def test_before_stop_false(self):
@@ -456,10 +503,11 @@ class TestWatcherHooks(TestCircus):
 
     @tornado.testing.gen_test
     def test_after_stop_fails(self):
-        with captured_output('stdout'):
-            yield self._test_hooks(behavior=ERROR, status='stopped',
-                                   hook_name='after_stop',
-                                   call=self._stop)
+        if captured_output:
+            with captured_output('stdout'):
+                yield self._test_hooks(behavior=ERROR, status='stopped',
+                                       hook_name='after_stop',
+                                       call=self._stop)
 
     @tornado.testing.gen_test
     def test_after_stop_false(self):
@@ -472,16 +520,39 @@ class TestWatcherHooks(TestCircus):
 
     @tornado.testing.gen_test
     def test_before_spawn_failure(self):
-        with captured_output('stdout'):
-            yield self._test_hooks(behavior=ERROR, status='stopped',
-                                   hook_name='before_spawn',
-                                   call=self._stop)
+        if captured_output:
+            with captured_output('stdout'):
+                yield self._test_hooks(behavior=ERROR, status='stopped',
+                                       hook_name='before_spawn',
+                                       call=self._stop)
 
     @tornado.testing.gen_test
     def test_before_spawn_false(self):
         yield self._test_hooks(behavior=FAILURE, status='stopped',
                                hook_name='before_spawn', call=self._stop)
 
+    @tornado.testing.gen_test
+    def test_after_spawn(self):
+        yield self._test_hooks(hook_name='after_spawn')
+
+    @tornado.testing.gen_test
+    def test_after_spawn_failure(self):
+        if captured_output:
+            with captured_output('stdout'):
+                yield self._test_hooks(behavior=ERROR, status='stopped',
+                                       hook_name='after_spawn',
+                                       call=self._stop)
+
+    @tornado.testing.gen_test
+    def test_after_spawn_false(self):
+        yield self._test_hooks(behavior=FAILURE, status='stopped',
+                               hook_name='after_spawn', call=self._stop)
+
+    @tornado.testing.gen_test
+    def test_extended_stats(self):
+        yield self._test_extended_stats()
+        yield self._test_extended_stats(extended=True)
+
 
 def oneshot_process(test_file):
     pass
@@ -510,7 +581,7 @@ class RespawnTest(TestCircus):
 
             # If we explicitely ask the watcher to respawn its processes,
             # ensure it's doing so.
-            yield watcher.spawn_processes()
+            yield watcher.start()
             self.assertEqual(len(watcher.processes), 1)
         finally:
             yield arbiter.stop()
