@@ -1,33 +1,55 @@
-import fcntl
-import grp
+import functools
 import logging
+import logging.config
 import os
-import pwd
 import re
 import shlex
 import socket
 import sys
 import time
-import functools
 import traceback
+import json
+try:
+    import yaml
+except ImportError:
+    yaml = None  # NOQA
+try:
+    import pwd
+    import grp
+    import fcntl
+except ImportError:
+    fcntl = None
+    grp = None
+    pwd = None
 from tornado.ioloop import IOLoop
 from tornado import gen
 from tornado import concurrent
-from circus.py3compat import integer_types, bytestring, raise_with_tb
+from circus.py3compat import (
+    integer_types, bytestring, raise_with_tb, text_type
+)
 try:
     from configparser import (
         ConfigParser, MissingSectionHeaderError, ParsingError, DEFAULTSECT
     )
-    new_config_parser = True
 except ImportError:
     from ConfigParser import (  # NOQA
         ConfigParser, MissingSectionHeaderError, ParsingError, DEFAULTSECT
     )
-    new_config_parser = False
+try:
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse  # NOQA
 
 from datetime import timedelta
 from functools import wraps
 import signal
+from pipes import quote as shell_escape_arg
+
+try:
+    import importlib
+    reload_module = importlib.reload
+except (ImportError, AttributeError):
+    from imp import reload as reload_module
 
 from zmq import ssh
 
@@ -69,24 +91,38 @@ LOG_LEVELS = {
     "info": logging.INFO,
     "debug": logging.DEBUG}
 
-LOG_FMT = r"%(asctime)s [%(process)d] [%(levelname)s] %(message)s"
+LOG_FMT = r"%(asctime)s %(name)s[%(process)d] [%(levelname)s] %(message)s"
 LOG_DATE_FMT = r"%Y-%m-%d %H:%M:%S"
+LOG_DATE_SYSLOG_FMT = r"%b %d %H:%M:%S"
 _SYMBOLS = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
 _all_signals = {}
 
 
 def get_working_dir():
-    """Returns current path, try to use PWD env first"""
+    """Returns current path, try to use PWD env first.
+
+    Since os.getcwd() resolves symlinks, we want to use
+    PWD first if present.
+    """
+    pwd_ = os.environ.get('PWD')
+    cwd = os.getcwd()
+
+    if pwd_ is None:
+        return cwd
+
+    # if pwd is the same physical file than the one
+    # pointed by os.getcwd(), we use it.
     try:
-        a = os.stat(os.environ['PWD'])
-        b = os.stat(os.getcwd())
-        if a.ino == b.ino and a.dev == b.dev:
-            working_dir = os.environ['PWD']
-        else:
-            working_dir = os.getcwd()
-    except:
-        working_dir = os.getcwd()
-    return working_dir
+        pwd_stat = os.stat(pwd_)
+        cwd_stat = os.stat(cwd)
+
+        if pwd_stat.ino == cwd_stat.ino and pwd_stat.dev == cwd_stat.dev:
+            return pwd_
+    except Exception:
+        pass
+
+    # otherwise, just use os.getcwd()
+    return cwd
 
 
 def bytes2human(n):
@@ -106,6 +142,48 @@ def bytes2human(n):
     return "%sB" % n
 
 
+_HSYMBOLS = {
+    'customary': ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
+    'customary_ext': ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa',
+                      'zetta', 'iotta'),
+    'iec': ('Bi', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'),
+    'iec_ext': ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi',
+                'zebi', 'yobi'),
+}
+
+
+_HSYMBOLS_VALUES = _HSYMBOLS.values()
+
+
+def human2bytes(s):
+    init = s
+    num = ""
+    while s and s[0:1].isdigit() or s[0:1] == '.':
+        num += s[0]
+        s = s[1:]
+
+    num = float(num)
+    letter = s.strip()
+
+    for sset in _HSYMBOLS_VALUES:
+        if letter in sset:
+            break
+
+    else:
+        if letter == 'k':
+            # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
+            sset = _HSYMBOLS['customary']
+            letter = letter.upper()
+        else:
+            raise ValueError("can't interpret %r" % init)
+
+    prefix = {sset[0]: 1}
+    for i, s in enumerate(sset[1:]):
+        prefix[s] = 1 << (i+1) * 10
+
+    return int(num * prefix[letter])
+
+
 # XXX weak dict ?
 _PROCS = {}
 
@@ -115,6 +193,12 @@ def get_info(process=None, interval=0, with_childs=False):
 
     If process is None, will return the information about the current process.
     """
+    # XXX moce get_info to circus.process ?
+    from circus.process import (get_children, get_memory_info,
+                                get_cpu_percent, get_memory_percent,
+                                get_cpu_times, get_nice, get_cmdline,
+                                get_create_time, get_username)
+
     if process is None or isinstance(process, int):
         if process is None:
             pid = os.getpid()
@@ -128,24 +212,24 @@ def get_info(process=None, interval=0, with_childs=False):
 
     info = {}
     try:
-        mem_info = process.get_memory_info()
+        mem_info = get_memory_info(process)
         info['mem_info1'] = bytes2human(mem_info[0])
         info['mem_info2'] = bytes2human(mem_info[1])
     except AccessDenied:
         info['mem_info1'] = info['mem_info2'] = "N/A"
 
     try:
-        info['cpu'] = process.get_cpu_percent(interval=interval)
+        info['cpu'] = get_cpu_percent(process, interval=interval)
     except AccessDenied:
         info['cpu'] = "N/A"
 
     try:
-        info['mem'] = round(process.get_memory_percent(), 1)
+        info['mem'] = round(get_memory_percent(process), 1)
     except AccessDenied:
         info['mem'] = "N/A"
 
     try:
-        cpu_times = process.get_cpu_times()
+        cpu_times = get_cpu_times(process)
         ctime = timedelta(seconds=sum(cpu_times))
         ctime = "%s:%s.%s" % (ctime.seconds // 60 % 60,
                               str((ctime.seconds % 60)).zfill(2),
@@ -161,32 +245,33 @@ def get_info(process=None, interval=0, with_childs=False):
         info['pid'] = 'N/A'
 
     try:
-        info['username'] = process.username
+        info['username'] = get_username(process)
     except AccessDenied:
         info['username'] = 'N/A'
 
     try:
-        try:
-            info['nice'] = process.get_nice()
-        except AttributeError:
-            info['nice'] = process.nice
+        info['nice'] = get_nice(process)
     except AccessDenied:
         info['nice'] = 'N/A'
     except NoSuchProcess:
         info['nice'] = 'Zombie'
 
+    raw_cmdline = get_cmdline(process)
+
     try:
-        cmdline = os.path.basename(shlex.split(process.cmdline[0])[0])
+        cmdline = os.path.basename(shlex.split(raw_cmdline[0])[0])
     except (AccessDenied, IndexError):
         cmdline = "N/A"
 
     try:
-        info['create_time'] = process.create_time
+        info['create_time'] = get_create_time(process)
     except AccessDenied:
         info['create_time'] = 'N/A'
 
     try:
-        info['age'] = time.time() - process.create_time
+        info['age'] = time.time() - get_create_time(process)
+    except TypeError:
+        info['create_time'] = get_create_time(process)
     except AccessDenied:
         info['age'] = 'N/A'
 
@@ -194,19 +279,22 @@ def get_info(process=None, interval=0, with_childs=False):
 
     info['children'] = []
     if with_childs:
-        for child in process.get_children():
+        for child in get_children(process):
             info['children'].append(get_info(child, interval=interval))
 
     return info
 
+TRUTHY_STRINGS = ('yes', 'true', 'on', '1')
+FALSY_STRINGS = ('no', 'false', 'off', '0')
+
 
 def to_bool(s):
     if isinstance(s, bool):
         return s
 
-    if s.lower().strip() in ("true", "1",):
+    if s.lower().strip() in TRUTHY_STRINGS:
         return True
-    elif s.lower().strip() in ("false", "0"):
+    elif s.lower().strip() in FALSY_STRINGS:
         return False
     else:
         raise ValueError("%r is not a boolean" % s)
@@ -230,63 +318,91 @@ def to_signum(signum):
         raise ValueError('signal invalid')
 
 
-def to_uid(name):
-    """Return an uid, given a user name.
-    If the name is an integer, make sure it's an existing uid.
+if pwd is None:
 
-    If the user name is unknown, raises a ValueError.
-    """
-    try:
-        name = int(name)
-    except ValueError:
-        pass
+    def to_uid(name):
+        raise RuntimeError("'to_uid' not available on this operating system")
 
-    if isinstance(name, int):
+else:
+
+    def to_uid(name):  # NOQA
+        """Return an uid, given a user name.
+        If the name is an integer, make sure it's an existing uid.
+
+        If the user name is unknown, raises a ValueError.
+        """
         try:
-            pwd.getpwuid(name)
-            return name
+            name = int(name)
+        except ValueError:
+            pass
+
+        if isinstance(name, int):
+            try:
+                pwd.getpwuid(name)
+                return name
+            except KeyError:
+                raise ValueError("%r isn't a valid user id" % name)
+
+        from circus.py3compat import string_types  # circular import fix
+
+        if not isinstance(name, string_types):
+            raise TypeError(name)
+
+        try:
+            return pwd.getpwnam(name).pw_uid
         except KeyError:
-            raise ValueError("%r isn't a valid user id" % name)
+            raise ValueError("%r isn't a valid user name" % name)
 
-    from circus.py3compat import string_types  # circular import fix
+if grp is None:
 
-    if not isinstance(name, string_types):
-        raise TypeError(name)
+    def to_gid(name):
+        raise RuntimeError("'to_gid' not available on this operating system")
 
-    try:
-        return pwd.getpwnam(name).pw_uid
-    except KeyError:
-        raise ValueError("%r isn't a valid user name" % name)
+else:
 
+    def to_gid(name):  # NOQA
+        """Return a gid, given a group name
 
-def to_gid(name):
-    """Return a gid, given a group name
-
-    If the group name is unknown, raises a ValueError.
-    """
-    try:
-        name = int(name)
-    except ValueError:
-        pass
-
-    if isinstance(name, int):
+        If the group name is unknown, raises a ValueError.
+        """
         try:
-            grp.getgrgid(name)
-            return name
-        # getgrid may raises overflow error on mac/os x, fixed in python2.7.5
-        # see http://bugs.python.org/issue17531
-        except (KeyError, OverflowError):
+            name = int(name)
+        except ValueError:
+            pass
+
+        if isinstance(name, int):
+            try:
+                grp.getgrgid(name)
+                return name
+            # getgrid may raises overflow error on mac/os x,
+            # fixed in python2.7.5
+            # see http://bugs.python.org/issue17531
+            except (KeyError, OverflowError):
+                raise ValueError("No such group: %r" % name)
+
+        from circus.py3compat import string_types  # circular import fix
+
+        if not isinstance(name, string_types):
+            raise TypeError(name)
+
+        try:
+            return grp.getgrnam(name).gr_gid
+        except KeyError:
             raise ValueError("No such group: %r" % name)
 
-    from circus.py3compat import string_types  # circular import fix
 
-    if not isinstance(name, string_types):
-        raise TypeError(name)
+def get_username_from_uid(uid):
+    """Return the username of a given uid."""
+    if isinstance(uid, int):
+        return pwd.getpwuid(uid).pw_name
+    return uid
 
-    try:
-        return grp.getgrnam(name).gr_gid
-    except KeyError:
-        raise ValueError("No such group: %r" % name)
+
+def get_default_gid(uid):
+    """Return the default group of a specific user."""
+    if isinstance(uid, int):
+        return pwd.getpwuid(uid).pw_gid
+    return pwd.getpwnam(uid).pw_gid
 
 
 def parse_env_str(env_str):
@@ -316,10 +432,18 @@ def env_to_str(env):
                      sorted(env.items(), key=lambda i: i[0])])
 
 
-def close_on_exec(fd):
-    flags = fcntl.fcntl(fd, fcntl.F_GETFD)
-    flags |= fcntl.FD_CLOEXEC
-    fcntl.fcntl(fd, fcntl.F_SETFD, flags)
+if fcntl is None:
+
+    def close_on_exec(fd):
+        raise RuntimeError(
+            "'close_on_exec' not available on this operating system")
+
+else:
+
+    def close_on_exec(fd):  # NOQA
+        flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+        flags |= fcntl.FD_CLOEXEC
+        fcntl.fcntl(fd, fcntl.F_SETFD, flags)
 
 
 def get_python_version():
@@ -412,7 +536,7 @@ class ImportStringError(ImportError):
                                  self.exception)
 
 
-def resolve_name(import_name, silent=False):
+def resolve_name(import_name, silent=False, reload=False):
     """Imports an object based on a string.  This is useful if you want to
     use import paths as endpoints or something similar.  An import path can
     be specified either in dotted notation (``xml.sax.saxutils.escape``)
@@ -423,6 +547,8 @@ def resolve_name(import_name, silent=False):
     :param import_name: the dotted name for the object to import.
     :param silent: if set to `True` import errors are ignored and
                    `None` is returned instead.
+    :param reload: if set to `True` modules that are already loaded will be
+                   reloaded
     :return: imported object
     """
     # force the import name to automatically convert to strings
@@ -430,20 +556,42 @@ def resolve_name(import_name, silent=False):
     try:
         if ':' in import_name:
             module, obj = import_name.split(':', 1)
-        elif '.' in import_name:
+        elif '.' in import_name and import_name not in sys.modules:
             module, obj = import_name.rsplit('.', 1)
         else:
-            return __import__(import_name)
+            module, obj = import_name, None
             # __import__ is not able to handle unicode strings in the fromlist
+
+        mod = None
         # if the module is a package
+        if reload and module in sys.modules:
+            try:
+                importlib.invalidate_caches()
+            except Exception:
+                pass
+            try:
+                mod = reload_module(sys.modules[module])
+            except Exception:
+                pass
+        if not mod:
+            if not obj:
+                return __import__(module)
+            try:
+                mod = __import__(module, None, None, [obj])
+            except ImportError:
+                if ':' in import_name:
+                    raise
+                return __import__(import_name)
+        if not obj:
+            return mod
         try:
-            return getattr(__import__(module, None, None, [obj]), obj)
-        except (ImportError, AttributeError):
+            return getattr(mod, obj)
+        except AttributeError:
             # support importing modules not yet set up by the parent module
             # (or package for that matter)
-            modname = module + '.' + obj
-            __import__(modname)
-            return sys.modules[modname]
+            if ':' in import_name:
+                raise
+            return __import__(import_name)
     except ImportError as e:
         if not silent:
             raise_with_tb(ImportStringError(import_name, e))
@@ -507,33 +655,71 @@ class ObjectDict(dict):
         return self[item]
 
 
-def configure_logger(logger, level='INFO', output="-"):
-    loglevel = LOG_LEVELS.get(level.lower(), logging.INFO)
-    logger.setLevel(loglevel)
-    if output == "-":
-        h = logging.StreamHandler()
+def configure_logger(logger, level='INFO', output="-", loggerconfig=None,
+                     name=None):
+    if loggerconfig is None or loggerconfig.lower().strip() == "default":
+        root_logger = logging.getLogger()
+        loglevel = LOG_LEVELS.get(level.lower(), logging.INFO)
+        root_logger.setLevel(loglevel)
+        datefmt = LOG_DATE_FMT
+        if output in ("-", "stdout"):
+            handler = logging.StreamHandler()
+        elif output.startswith('syslog://'):
+            # URLs are syslog://host[:port]?facility or syslog:///path?facility
+            info = urlparse(output)
+            facility = 'user'
+            if info.query in logging.handlers.SysLogHandler.facility_names:
+                facility = info.query
+            if info.netloc:
+                address = (info.netloc, info.port or 514)
+            else:
+                address = info.path
+            datefmt = LOG_DATE_SYSLOG_FMT
+            handler = logging.handlers.SysLogHandler(
+                address=address, facility=facility)
+        else:
+            handler = logging.handlers.WatchedFileHandler(output)
+            close_on_exec(handler.stream.fileno())
+        formatter = logging.Formatter(fmt=LOG_FMT, datefmt=datefmt)
+        handler.setFormatter(formatter)
+        root_logger.handlers = [handler]
     else:
-        h = logging.FileHandler(output)
-        close_on_exec(h.stream.fileno())
-    fmt = logging.Formatter(LOG_FMT, LOG_DATE_FMT)
-    h.setFormatter(fmt)
-    logger.handlers = [h]
-    logger.propagate = False
+        loggerconfig = os.path.abspath(loggerconfig)
+        if loggerconfig.lower().endswith(".ini"):
+            logging.config.fileConfig(loggerconfig,
+                                      disable_existing_loggers=True)
+        elif loggerconfig.lower().endswith(".json"):
+            if not hasattr(logging.config, "dictConfig"):
+                raise Exception("Logger configuration file %s appears to be "
+                                "a JSON file but this version of Python "
+                                "does not support the "
+                                "logging.config.dictConfig function. Try "
+                                "Python 2.7.")
+            with open(loggerconfig, "r") as fh:
+                logging.config.dictConfig(json.loads(fh.read()))
+        elif loggerconfig.lower().endswith(".yaml"):
+            if not hasattr(logging.config, "dictConfig"):
+                raise Exception("Logger configuration file %s appears to be "
+                                "a YAML file but this version of Python "
+                                "does not support the "
+                                "logging.config.dictConfig function. Try "
+                                "Python 2.7.")
+            if yaml is None:
+                raise Exception("Logger configuration file %s appears to be "
+                                "a YAML file but PyYAML is not available. "
+                                "Try: pip install PyYAML"
+                                % (shell_escape_arg(loggerconfig),))
+            with open(loggerconfig, "r") as fh:
+                logging.config.dictConfig(yaml.load(fh.read()))
+        else:
+            raise Exception("Logger configuration file %s is not in one "
+                            "of the recognized formats.  The file name "
+                            "should be: *.ini, *.json or *.yaml."
+                            % (shell_escape_arg(loggerconfig),))
 
 
 class StrictConfigParser(ConfigParser):
 
-    if new_config_parser:
-        def toboolean(self, value):
-            if value.lower() not in self.BOOLEAN_STATES:
-                raise ValueError('Not a boolean: %s' % value)
-            return self.BOOLEAN_STATES[value.lower()]
-    else:
-        def toboolean(self, value):  # NOQA
-            if value.lower() not in self._boolean_states:
-                raise ValueError('Not a boolean: %s' % value)
-            return self._boolean_states[value.lower()]
-
     def _read(self, fp, fpname):
         cursect = None                        # None, or a dictionary
         optname = None
@@ -583,6 +769,7 @@ class StrictConfigParser(ConfigParser):
                         mo = self.OPTCRE.match(line)    # 2.6
                     if mo:
                         optname, vi, optval = mo.group('option', 'vi', 'value')
+                        self.optionxform = text_type
                         optname = self.optionxform(optname.rstrip())
                         # We don't want to override.
                         if optname in cursect:
@@ -654,6 +841,11 @@ def load_virtualenv(watcher):
     if not os.path.exists(sitedir):
         raise ValueError("%s does not exist" % sitedir)
 
+    bindir = os.path.join(watcher.virtualenv, 'bin')
+
+    if os.path.exists(bindir):
+        watcher.env['PATH'] = ':'.join([bindir, watcher.env.get('PATH', '')])
+
     def process_pth(sitedir, name):
         packages = set()
         fullname = os.path.join(sitedir, name)
@@ -694,7 +886,7 @@ def load_virtualenv(watcher):
             path = venv_path
 
     # Add watcher virtualenv site-packages dir to the python path
-    if path and not sitedir in path.split(os.pathsep):
+    if path and sitedir not in path.split(os.pathsep):
         path = os.pathsep.join([path, sitedir])
     else:
         if py_path:
@@ -726,7 +918,11 @@ def create_udp_socket(mcast_addr, mcast_port):
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     # Some platform exposes SO_REUSEPORT
     if hasattr(socket, 'SO_REUSEPORT'):
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
+        try:
+            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
+        except socket.error:
+            # see #699
+            pass
     # Put packet ttl to max
     sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 255)
     # Register socket to multicast group
@@ -789,6 +985,8 @@ def synchronized(name):
             elif hasattr(self, "_exclusive_running_command"):
                 arbiter = self
             if arbiter is not None:
+                if arbiter._restarting:
+                    raise ConflictError("arbiter is restarting...")
                 if arbiter._exclusive_running_command is not None:
                     raise ConflictError("arbiter is already running %s command"
                                         % arbiter._exclusive_running_command)
@@ -864,3 +1062,8 @@ def check_future_exception_and_log(future):
                 exc_info = future.exc_info()
                 traceback.print_tb(exc_info[2])
             return exception
+
+
+def is_win():
+    """checks if platform is Windows"""
+    return sys.platform == "win32"
