@@ -14,7 +14,6 @@ from tornado import gen
 
 from psutil import NoSuchProcess
 import zmq.utils.jsonapi as json
-from zmq.utils.strtypes import b
 from zmq.eventloop import ioloop
 
 from circus.process import Process, DEAD_OR_ZOMBIE, UNEXISTING
@@ -22,7 +21,7 @@ from circus import logger
 from circus import util
 from circus.stream import get_pipe_redirector, get_stream
 from circus.util import parse_env_dict, resolve_name, tornado_sleep
-from circus.py3compat import bytestring, is_callable
+from circus.py3compat import bytestring, is_callable, b
 
 
 class Watcher(object):
@@ -81,10 +80,11 @@ class Watcher(object):
         `circus.stream.FileStream`
       - **filename**: the filename, if using a FileStream
       - **max_bytes**: maximum file size, after which a new output file is
-        opened. defaults to 0 which means no maximum size.
+        opened. defaults to 0 which means no maximum size (only applicable
+        with FileStream).
       - **backup_count**: how many backups to retain when rotating files
         according to the max_bytes parameter. defaults to 0 which means
-        no backups are made.
+        no backups are made (only applicable with FileStream)
 
       This mapping will be used to create a stream callable of the specified
       class.
@@ -102,10 +102,11 @@ class Watcher(object):
       - **class**: the stream class. Defaults to `circus.stream.FileStream`
       - **filename**: the filename, if using a FileStream
       - **max_bytes**: maximum file size, after which a new output file is
-        opened. defaults to 0 which means no maximum size.
+        opened. defaults to 0 which means no maximum size (only applicable
+        with FileStream)
       - **backup_count**: how many backups to retain when rotating files
         according to the max_bytes parameter. defaults to 0 which means
-        no backups are made.
+        no backups are made (only applicable with FileStream).
 
       This mapping will be used to create a stream callable of the specified
       class.
@@ -153,8 +154,8 @@ class Watcher(object):
       or the callabled itself and a boolean flag indicating if an
       exception occuring in the hook should not be ignored.
       Possible values for the hook name: *before_start*, *after_start*,
-      *before_spawn*, *before_stop*, *after_stop*., *before_signal* or
-      *after_signal*.
+      *before_spawn*, *after_spawn*, *before_stop*, *after_stop*.,
+      *before_signal*, *after_signal* or *extended_stats*.
 
     - **options** -- extra options for the worker. All options
       found in the configuration file for instance, are passed
@@ -175,12 +176,12 @@ class Watcher(object):
     """
 
     def __init__(self, name, cmd, args=None, numprocesses=1, warmup_delay=0.,
-                 working_dir=None, shell=False, uid=None, max_retry=5,
-                 gid=None, send_hup=False, stop_signal=signal.SIGTERM,
-                 stop_children=False, env=None, graceful_timeout=30.0,
-                 prereload_fn=None, rlimits=None, executable=None,
-                 stdout_stream=None, stderr_stream=None, priority=0,
-                 loop=None, singleton=False, use_sockets=False,
+                 working_dir=None, shell=False, shell_args=None, uid=None,
+                 max_retry=5, gid=None, send_hup=False,
+                 stop_signal=signal.SIGTERM, stop_children=False, env=None,
+                 graceful_timeout=30.0, prereload_fn=None, rlimits=None,
+                 executable=None, stdout_stream=None, stderr_stream=None,
+                 priority=0, loop=None, singleton=False, use_sockets=False,
                  copy_env=False, copy_path=False, max_age=0,
                  max_age_variance=30, hooks=None, respawn=True,
                  autostart=True, on_demand=False, virtualenv=None,
@@ -213,7 +214,8 @@ class Watcher(object):
         self.max_age = int(max_age)
         self.max_age_variance = int(max_age_variance)
         self.ignore_hook_failure = ['before_stop', 'after_stop',
-                                    'before_signal', 'after_signal']
+                                    'before_signal', 'after_signal',
+                                    'extended_stats']
 
         self.respawn = respawn
         self.autostart = autostart
@@ -227,9 +229,10 @@ class Watcher(object):
 
         self.optnames = (("numprocesses", "warmup_delay", "working_dir",
                           "uid", "gid", "send_hup", "stop_signal",
-                          "stop_children", "shell", "env", "max_retry", "cmd",
-                          "args", "graceful_timeout", "executable",
-                          "use_sockets", "priority", "copy_env", "singleton",
+                          "stop_children", "shell", "shell_args",
+                          "env", "max_retry", "cmd", "args",
+                          "graceful_timeout", "executable", "use_sockets",
+                          "priority", "copy_env", "singleton",
                           "stdout_stream_conf", "on_demand",
                           "stderr_stream_conf", "max_age", "max_age_variance",
                           "close_child_stdout", "close_child_stderr")
@@ -242,6 +245,7 @@ class Watcher(object):
         self.working_dir = working_dir
         self.processes = {}
         self.shell = shell
+        self.shell_args = shell_args
         self.uid = uid
         self.gid = gid
 
@@ -280,26 +284,44 @@ class Watcher(object):
 
     def _reload_hook(self, key, hook, ignore_error):
         hook_name = key.split('.')[-1]
-        self._resolve_hook(hook_name, hook, ignore_error)
+        self._resolve_hook(hook_name, hook, ignore_error, reload_module=True)
 
     def _reload_stream(self, key, val):
         parts = key.split('.', 1)
 
-        if parts[0] == 'stdout':
+        action = 0
+        if parts[0] == 'stdout_stream':
+            old_stream = self.stdout_stream
             self.stdout_stream_conf[parts[1]] = val
-            self.stdout_stream = get_stream(self.stdout_stream_conf)
+            self.stdout_stream = get_stream(self.stdout_stream_conf,
+                                            reload=True)
+            if self.stdout_redirector:
+                self.stdout_redirector.redirect = self.stdout_stream['stream']
+            else:
+                self.stdout_redirector = get_pipe_redirector(
+                    self.stdout_stream, loop=self.loop)
+                self.stdout_redirector.start()
+                action = 1
+
+            if old_stream and hasattr(old_stream['stream'], 'close'):
+                old_stream['stream'].close()
         else:
+            old_stream = self.stderr_stream
             self.stderr_stream_conf[parts[1]] = val
-            self.stderr_stream = get_stream(self.stderr_stream_conf)
+            self.stderr_stream = get_stream(self.stderr_stream_conf,
+                                            reload=True)
+            if self.stderr_redirector:
+                self.stderr_redirector.redirect = self.stderr_stream['stream']
+            else:
+                self.stderr_redirector = get_pipe_redirector(
+                    self.stderr_stream, loop=self.loop)
+                self.stderr_redirector.start()
+                action = 1
 
-        self._create_redirectors()
-        if self.stdout_redirector is not None:
-            self.stdout_redirector.start()
+            if old_stream and hasattr(old_stream['stream'], 'close'):
+                old_stream['stream'].close()
 
-        if self.stderr_redirector is not None:
-            self.stderr_redirector.start()
-
-        return 1
+        return action
 
     def _create_redirectors(self):
         if self.stdout_stream:
@@ -318,12 +340,14 @@ class Watcher(object):
         else:
             self.stderr_redirector = None
 
-    def _resolve_hook(self, name, callable_or_name, ignore_failure):
+    def _resolve_hook(self, name, callable_or_name, ignore_failure,
+                      reload_module=False):
         if is_callable(callable_or_name):
             self.hooks[name] = callable_or_name
         else:
             # will raise ImportError on failure
-            self.hooks[name] = resolve_name(callable_or_name)
+            self.hooks[name] = resolve_name(callable_or_name,
+                                            reload=reload_module)
 
         if ignore_failure:
             self.ignore_hook_failure.append(name)
@@ -336,6 +360,10 @@ class Watcher(object):
         for name, (callable_or_name, ignore_failure) in hooks.items():
             self._resolve_hook(name, callable_or_name, ignore_failure)
 
+    @property
+    def pending_socket_event(self):
+        return self.on_demand and not self.arbiter.socket_event
+
     @classmethod
     def load_from_config(cls, config):
         if 'env' in config:
@@ -373,7 +401,7 @@ class Watcher(object):
             return
         process = self.processes.pop(pid)
 
-        if not status:
+        if status is None:
             while True:
                 try:
                     _, status = os.waitpid(pid, os.WNOHANG)
@@ -385,11 +413,16 @@ class Watcher(object):
                         # nothing to do here, we do not have any child
                         # process running
                         # but we still need to send the "reap" signal.
+                        #
+                        # This can happen if poll() or wait() were called on
+                        # the underlying process.
                         logger.debug('reaping already dead process %s [%s]',
                                      pid, self.name)
                         self.notify_event(
                             "reap",
-                            {"process_pid": pid, "time": time.time()})
+                            {"process_pid": pid,
+                             "time": time.time(),
+                             "exit_code": process.returncode()})
                         process.stop()
                         return
                     else:
@@ -397,11 +430,18 @@ class Watcher(object):
 
         # get return code
         if os.WIFSIGNALED(status):
-            os.WTERMSIG(status)
+            # The Python Popen object returns <-signal> in it's returncode
+            # property if the process exited on a signal, so emulate that
+            # behavior here so that pubsub clients watching for reap can
+            # distinguish between an exit with a non-zero exit code and
+            # a signal'd exit. This is also consistent with the notify_event
+            # reap message above that uses the returncode function (that ends
+            # up calling Popen.returncode)
+            exit_code = -os.WTERMSIG(status)
         # process exited using exit(2) system call; return the
         # integer exit(2) system call has been called with
         elif os.WIFEXITED(status):
-            os.WEXITSTATUS(status)
+            exit_code = os.WEXITSTATUS(status)
         else:
             # should never happen
             raise RuntimeError("Unknown process exit status")
@@ -411,7 +451,10 @@ class Watcher(object):
             process.stop()
 
         logger.debug('reaping process %s [%s]', pid, self.name)
-        self.notify_event("reap", {"process_pid": pid, "time": time.time()})
+        self.notify_event("reap",
+                          {"process_pid": pid,
+                           "time": time.time(),
+                           "exit_code": exit_code})
 
     @util.debuglog
     def reap_processes(self):
@@ -434,16 +477,18 @@ class Watcher(object):
 
         # remove dead or zombie processes first
         for process in list(self.processes.values()):
-            if process.status == DEAD_OR_ZOMBIE:
+            if process.status in (DEAD_OR_ZOMBIE, UNEXISTING):
                 self.processes.pop(process.pid)
 
         if self.max_age:
             yield self.remove_expired_processes()
 
         # adding fresh processes
-        if (self.respawn and len(self.processes) < self.numprocesses
-                and not self.is_stopping()):
-            yield self.spawn_processes()
+        if len(self.processes) < self.numprocesses and not self.is_stopping():
+            if self.respawn:
+                yield self.spawn_processes()
+            elif not len(self.processes) and not self.on_demand:
+                yield self._stop()
 
         # removing extra processes
         if len(self.processes) > self.numprocesses:
@@ -451,7 +496,7 @@ class Watcher(object):
             for process in sorted(self.processes.values(),
                                   key=lambda process: process.started,
                                   reverse=True)[self.numprocesses:]:
-                if process.status == DEAD_OR_ZOMBIE:
+                if process.status in (DEAD_OR_ZOMBIE, UNEXISTING):
                     self.processes.pop(process.pid)
                 else:
                     processes_to_kill.append(process)
@@ -489,7 +534,7 @@ class Watcher(object):
         """
         # when an on_demand process dies, do not restart it until
         # the next event
-        if self.on_demand and not self.arbiter.socket_event:
+        if self.pending_socket_event:
             self._status = "stopped"
             return
         for i in range(self.numprocesses - len(self.processes)):
@@ -553,6 +598,10 @@ class Watcher(object):
                 self.processes[process.pid] = process
                 logger.debug('running %s process [pid %d]', self.name,
                              process.pid)
+                if not self.call_hook('after_spawn', pid=process.pid):
+                    self.kill_process(process)
+                    del self.processes[process.pid]
+                    return False
             except OSError as e:
                 logger.warning('error in %r: %s', self.name, str(e))
 
@@ -571,16 +620,22 @@ class Watcher(object):
 
         The signal is sent to the process itself then to all the children
         """
+        children = None
         try:
+            # getting the process children
+            children = process.children()
+
             # sending the signal to the process itself
             self.send_signal(process.pid, signum)
             self.notify_event("kill", {"process_pid": process.pid,
                                        "time": time.time()})
         except NoSuchProcess:
             # already dead !
-            pass
+            if children is None:
+                return
+
         # now sending the same signal to all the children
-        for child_pid in process.children():
+        for child_pid in children:
             try:
                 process.send_signal_child(child_pid, signum)
                 self.notify_event("kill", {"process_pid": child_pid,
@@ -604,13 +659,17 @@ class Watcher(object):
         """
         if process.stopping:
             raise gen.Return(False)
-        logger.debug("%s: kill process %s", self.name, process.pid)
-        if self.stop_children:
-            self.send_signal_process(process, self.stop_signal)
-        else:
-            self.send_signal(process.pid, self.stop_signal)
-            self.notify_event("kill", {"process_pid": process.pid,
-                                       "time": time.time()})
+        try:
+            logger.debug("%s: kill process %s", self.name, process.pid)
+            if self.stop_children:
+                self.send_signal_process(process, self.stop_signal)
+            else:
+                self.send_signal(process.pid, self.stop_signal)
+                self.notify_event("kill", {"process_pid": process.pid,
+                                           "time": time.time()})
+        except NoSuchProcess:
+            raise gen.Return(False)
+
         process.stopping = True
         waited = 0
         while waited < self.graceful_timeout:
@@ -676,14 +735,25 @@ class Watcher(object):
         return self._status
 
     @util.debuglog
-    def process_info(self, pid):
+    def process_info(self, pid, extended=False):
         process = self.processes[int(pid)]
-        return process.info()
+        result = process.info()
+        if extended and 'extended_stats' in self.hooks:
+            self.hooks['extended_stats'](self, self.arbiter,
+                                         'extended_stats',
+                                         pid=pid, stats=result)
+        return result
 
     @util.debuglog
-    def info(self):
-        return dict([(proc.pid, proc.info())
-                     for proc in self.processes.values()])
+    def info(self, extended=False):
+        result = dict([(proc.pid, proc.info())
+                       for proc in self.processes.values()])
+        if extended and 'extended_stats' in self.hooks:
+            for pid, stats in result.items():
+                self.hooks['extended_stats'](self, self.arbiter,
+                                             'extended_stats',
+                                             pid=pid, stats=stats)
+        return result
 
     @util.synchronized("watcher_stop")
     @gen.coroutine
@@ -692,7 +762,7 @@ class Watcher(object):
 
     @util.debuglog
     @gen.coroutine
-    def _stop(self):
+    def _stop(self, close_output_streams=False):
         if self.is_stopped():
             return
         self._status = "stopping"
@@ -702,6 +772,8 @@ class Watcher(object):
         # We ignore the hook result
         self.call_hook('before_stop')
         yield self.kill_processes()
+        self.reap_processes()
+
         # stop redirectors
         if self.stdout_redirector is not None:
             self.stdout_redirector.stop()
@@ -709,6 +781,13 @@ class Watcher(object):
         if self.stderr_redirector is not None:
             self.stderr_redirector.stop()
             self.stderr_redirector = None
+        if close_output_streams:
+            if self.stdout_stream and hasattr(self.stdout_stream['stream'],
+                                              'close'):
+                self.stdout_stream['stream'].close()
+            if self.stderr_stream and hasattr(self.stderr_stream['stream'],
+                                              'close'):
+                self.stderr_stream['stream'].close()
         # notify about the stop
         if self.evpub_socket is not None:
             self.notify_event("stop", {"time": time.time()})
@@ -766,17 +845,24 @@ class Watcher(object):
     @util.synchronized("watcher_start")
     @gen.coroutine
     def start(self):
+        before_pids = set() if self.is_stopped() else set(self.processes)
         yield self._start()
+        after_pids = set(self.processes)
+        raise gen.Return({'started': sorted(after_pids - before_pids),
+                          'kept': sorted(after_pids & before_pids)})
 
     @gen.coroutine
     @util.debuglog
     def _start(self):
         """Start.
         """
-        if not self.is_stopped():
+        if self.pending_socket_event:
             return
 
-        if self.on_demand and not self.arbiter.socket_event:
+        if not self.is_stopped():
+            if len(self.processes) < self.numprocesses:
+                self.reap_processes()
+                yield self.spawn_processes()
             return
 
         if not self.call_hook('before_start'):
@@ -789,7 +875,9 @@ class Watcher(object):
         self.reap_processes()
         yield self.spawn_processes()
 
-        if not self.call_hook('after_start'):
+        # If not self.processes, the before_spawn or after_spawn hooks have
+        # probably prevented startup so give up
+        if not self.processes or not self.call_hook('after_start'):
             logger.debug('Aborting startup')
             yield self._stop()
             return
@@ -807,7 +895,12 @@ class Watcher(object):
     @util.synchronized("watcher_restart")
     @gen.coroutine
     def restart(self):
+        before_pids = set() if self.is_stopped() else set(self.processes)
         yield self._restart()
+        after_pids = set(self.processes)
+        raise gen.Return({'stopped': sorted(before_pids - after_pids),
+                          'started': sorted(after_pids - before_pids),
+                          'kept': sorted(after_pids & before_pids)})
 
     @gen.coroutine
     @util.debuglog
@@ -817,14 +910,21 @@ class Watcher(object):
 
     @util.synchronized("watcher_reload")
     @gen.coroutine
-    def reload(self, graceful=True):
-        yield self._reload(graceful=graceful)
+    def reload(self, graceful=True, sequential=False):
+        before_pids = set() if self.is_stopped() else set(self.processes)
+        yield self._reload(graceful=graceful, sequential=sequential)
+        after_pids = set(self.processes)
+        raise gen.Return({'stopped': sorted(before_pids - after_pids),
+                          'started': sorted(after_pids - before_pids),
+                          'kept': sorted(after_pids & before_pids)})
 
     @gen.coroutine
     @util.debuglog
-    def _reload(self, graceful=True):
+    def _reload(self, graceful=True, sequential=False):
         """ reload
         """
+        if not(graceful) and sequential:
+            logger.warn("with graceful=False, sequential=True is ignored")
         if self.prereload_fn is not None:
             self.prereload_fn(self)
 
@@ -832,14 +932,24 @@ class Watcher(object):
             yield self._restart()
             return
 
-        if self.send_hup:
+        if self.is_stopped():
+            yield self._start()
+        elif self.send_hup:
             for process in self.processes.values():
                 logger.info("SENDING HUP to %s" % process.pid)
                 process.send_signal(signal.SIGHUP)
         else:
-            for i in range(self.numprocesses):
-                self.spawn_process()
-            yield self.manage_processes()
+            if sequential:
+                active_processes = self.get_active_processes()
+                for process in active_processes:
+                    yield self.kill_process(process)
+                    self.reap_process(process.pid)
+                    self.spawn_process()
+                    yield tornado_sleep(self.warmup_delay)
+            else:
+                for i in range(self.numprocesses):
+                    self.spawn_process()
+                yield self.manage_processes()
         self.notify_event("reload", {"time": time.time()})
         logger.info('%s reloaded', self.name)
 
@@ -930,7 +1040,7 @@ class Watcher(object):
         elif (key.startswith('stdout_stream') or
               key.startswith('stderr_stream')):
             action = self._reload_stream(key, val)
-        elif key.startswith('hook'):
+        elif key.startswith('hooks'):
             val = val.split(',')
             if len(val) == 2:
                 ignore_error = util.to_bool(val[1])
@@ -938,7 +1048,7 @@ class Watcher(object):
                 ignore_error = False
             hook = val[0]
             self._reload_hook(key, hook, ignore_error)
-            action = 1
+            action = 0
 
         # send update event
         self.notify_event("updated", {"time": time.time()})
@@ -950,9 +1060,9 @@ class Watcher(object):
         # trigger needed action
         if num == 0:
             yield self.manage_processes()
-        else:
+        elif not self.is_stopped():
             # graceful restart
-            yield self._restart()
+            yield self._reload()
 
     @util.debuglog
     def options(self, *args):
